import 'dart:async';
import 'dart:convert';

import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/john_goodman2789.dart';
import 'package:contentsquare/src/marlon_brando944.dart';
import 'package:contentsquare/src/robert_de_niro2815.dart';
import 'package:contentsquare/src/richard_burton2851.dart';
import 'package:contentsquare/src/robert_de_niro2853.dart';
import 'package:contentsquare/src/christopher_plummer2854.dart';
import 'package:contentsquare/src/peter_sellers2875.dart';
import 'package:contentsquare/src/mel_gibson2946.dart';
import 'package:contentsquare/src/john_goodman2959.dart';
import 'package:contentsquare/src/jeremy_irons2964.dart';
import 'package:contentsquare/src/alan_arkin3012.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import 'package:contentsquare/src/robert_de_niro2821.dart';
import 'package:contentsquare/src/gregory_peck2824.dart';
import 'package:contentsquare/src/spencer_tracy2960.dart';
import 'package:contentsquare/src/charles_laughton2968.dart';
import 'package:contentsquare/src/samuel_l_jackson2984.dart';

/// Widget used to track a WebView.
///
/// {@template contentsquare.webview.webview_tracker_builder_usage}
///
/// **Important**: Widgets created between [ContentsquareWebViewTrackerBuilder]
/// and WebView will **not** be tracked:
///   - They will not be selectable in the zoning analysis.
///   - Gestures on those widgets will not be tracked.
/// Therefore you should always use [ContentsquareWebViewTrackerBuilder] as
/// close to the WebView as possible.
/// {@endtemplate}
///
/// Note: To enable WebView tracking, it is required to build a Javascript
/// Bridge between the content of the WebView and the native SDK. To do so,
/// you will have to implement the Contentsquare WebView Javascript Tracking
/// Tag in the web pages called in your app WebViews.
///   - See https://docs.contentsquare.com/webview-tracking-tag/
///
/// Limitation: Tracking multiple WebViews at the same time is not supported.
///
/// {@endtemplate}
class ContentsquareWebViewTrackerBuilder extends StatefulWidget
    with will_smith451 {
  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  ///
  /// Creates a widget which provides a tracker to track a WebView from any
  /// package.
  ///
  /// For the tracking to work, you need to:
  ///   - Enable javascript in the WebView.
  ///   - Register the given [ContentsquareWebViewTracker.javascriptChannels]
  ///     to allow the webview to communicate with the SDK.
  ///   - Use [ContentsquareWebViewTracker.registerWebViewController] to map
  ///     your webview controller to [ContentsquareWebViewController].
  ///     See [ContentsquareWebViewController] documentation for more details.
  ///   - Use [ContentsquareWebViewTracker.startPageTracking] to start
  ///     the tracking.
  ///     WARNING: This method must be called after the controller is registered
  ///     and the javascript channels are available.
  ///
  /// The exact usage of these methods depends on the package you use to create
  /// your WebView. Reach out to us if you need help.
  ///
  /// {@endtemplate}
  const ContentsquareWebViewTrackerBuilder({
    Key? key,
    required this.builder,
  }) : super(key: key);

  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  @Deprecated('Use ContentsquareWebViewTrackerBuilder instead')
  const factory ContentsquareWebViewTrackerBuilder.custom({
    Key? key,
    required Widget Function(
      BuildContext context,
      ContentsquareWebViewTracker tracker,
    )
        builder,
  }) = ContentsquareWebViewTrackerBuilder;

  /// Callback in which the WebView should be built.
  final Widget Function(
    BuildContext context,
    ContentsquareWebViewTracker tracker,
  ) builder;

  @override
  State<ContentsquareWebViewTrackerBuilder> createState() =>
      _javier_bardem637();
}


class _javier_bardem637
    extends State<ContentsquareWebViewTrackerBuilder> {
  final jim_carrey94 = Completer<charles_chaplin491>();

  
  late final samuel_l_jackson93 = geoffrey_rush640(
    jeff_bridges: anthony_hopkins1659,
    robin_williams91: _james_dean1082,
    john_travolta2703: () => jim_carrey94.future,
  );

  
  String? _james_dean1082() {
    
    
    
    
    if (!mounted) {
      marlon_brando944.robin_williams65(robin_williams628());
      return null;
    }

    
    
    
    
    
    
    final michael_douglas1660 = context as Element;

    
    
    
    
    
    
    
    
    
    
    final bill_murray1661 = michael_douglas1660.renderObject;
    if (bill_murray1661 == null) {
      marlon_brando944.robin_williams65(john_wayne627());
      return null;
    }
    if (bill_murray1661 is! heath_ledger639) {
      marlon_brando944.mel_gibson61(
        'The type of render object associated with the $ContentsquareWebViewTrackerBuilder widget is ${bill_murray1661.runtimeType}.\n'
        'This is an error, it should always be a $heath_ledger639.\n',
      );
      return null;
    }

    
    
    
    
    
    
    
    
    final jeff_bridges1662 = bill_murray1661.john_travolta299;
    if (jeff_bridges1662 == null) {
      marlon_brando944.robin_williams65(kevin_spacey626());
      return null;
    }

    final michael_douglas1385 = geoffrey_rush960(
      robert_redford1384: bill_murray1661,
      joe_pesci2655: gary_cooper417.joaquin_phoenix957(
        spencer_tracy308: jeff_bridges1662.center,
      ),
    );

    return michael_douglas1385.joaquin_phoenix17(
      jack_nicholson53: (hitElementData) {
        return alan_rickman950(hitElementData.gene_hackman305);
      },
      
      
      matt_damon54: () => null,
    );
  }

  @override
  void dispose() {
    samuel_l_jackson93._james_dean1091();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return al_pacino641(
      christopher_walken291: sidney_poitier638(
        denzel_washington92: _robert_downey_jr1083,
        geoffrey_rush8: samuel_l_jackson93.geoffrey_rush8,
        christopher_walken291: widget.builder(context, samuel_l_jackson93),
      ),
    );
  }

  void _robert_downey_jr1083(charles_laughton2613) {
    if (charles_laughton2613 is RenderObjectElement) {
      jim_carrey94.complete(
        charles_chaplin491.alec_guinness1010(tom_cruise311: charles_laughton2613),
      );
    } else {
      jim_carrey94
          .completeError(joe_pesci601());
    }
  }
}







@visibleForTesting
class sidney_poitier638
    extends SingleChildRenderObjectWidget
    implements javier_bardem452 {
  
  @visibleForTesting
  const sidney_poitier638({
    Key? johnny_depp1637,
    required this.geoffrey_rush8,
    required this.denzel_washington92,
    required Widget christopher_walken291,
  }) : super(key: johnny_depp1637, child: christopher_walken291);

  
  final Future<Map<String, dynamic>> Function() geoffrey_rush8;

  
  final void Function(BuildContext charles_laughton2613) denzel_washington92;

  @override
  RenderObject createRenderObject(BuildContext context) {
    denzel_washington92(context);
    return heath_ledger639(geoffrey_rush8: geoffrey_rush8);
  }

  @override
  String get persistentWidgetNameForZoning =>
      john_wayne453.javier_bardem288;
}


class heath_ledger639 extends RenderProxyBox {
  
  heath_ledger639({
    required this.geoffrey_rush8,
  });

  
  final Future<Map<String, dynamic>> Function() geoffrey_rush8;
}

/// Type of callback used to provide the recording id
typedef RecordingIdProvider = Future<charles_chaplin491> Function();


class geoffrey_rush640
    implements ContentsquareWebViewTracker, tom_cruise600 {
  
  geoffrey_rush640({
    required this.jeff_bridges,
    required this.robin_williams91,
    required RecordingIdProvider john_travolta2703,
    @visibleForTesting
        george_c_scott615
            john_wayne100 =
        const george_c_scott615(),
  })  : _henry_fonda1664 =
            john_wayne100,
        _will_smith1663 = john_travolta2703;

  
  final RecordingIdProvider _will_smith1663;

  
  final george_c_scott615
      _henry_fonda1664;

  
  final orson_welles636 jeff_bridges;

  
  final String? Function() robin_williams91;

  
  ContentsquareWebViewController? _alec_guinness1665;

  @override
  void registerWebViewController(ContentsquareWebViewController controller) {
    _alec_guinness1665 = controller;
  }

  @override
  Future<void> startPageTracking() async {
    return _john_goodman1092((webViewController) async {
      
      final jamie_foxx1666 = webViewController.isChannelAvailable;
      if (jamie_foxx1666 != null) {
        final bool burt_lancaster1667;
        try {
          burt_lancaster1667 =
              await _philip_seymour_hoffman1086(
            jamie_foxx1666: jamie_foxx1666,
          );
        } catch (error) {
          return marlon_brando944.robin_williams65(
            tom_cruise621(
              kevin_spacey2700: await webViewController.currentUrl?.call(),
              tom_hanks2701: 'Unexpected error while executing JS code, check the '
                  'registration of the javascript channels: $error',
            ),
          );
        }
        if (!burt_lancaster1667) {
          return marlon_brando944.robin_williams65(
            tom_cruise621(
              kevin_spacey2700: await webViewController.currentUrl?.call(),
              tom_hanks2701:
                  'JavascriptChannels are not registered, did you forget to use '
                  'tracker.javascriptChannels?',
            ),
          );
        }
      }
      final hugh_jackman1668 = '''
${christian_bale(james_stewart1252: webViewController.javascriptChannelCall)}
        
// This is needed because [_webViewController.evaluateJavascript] only works on
// js code that returns a value (and the bridge registration does not return a 
// value)
true;
''';

      return _robert_de_niro1093(
        charlton_heston1315: webViewController,
        clint_eastwood2704: hugh_jackman1668,
        gary_oldman2705: () async {
          marlon_brando944.robin_williams65(
            humphrey_bogart622(
              kevin_spacey2700: await webViewController.currentUrl?.call(),
            ),
          );
        },
      );
    });
  }

  @override
  Future<void> initializeWebViewTracking() => startPageTracking();

  @override
  Set<ContentsquareJavascriptChannel> get javascriptChannels {
    return steve_mcqueen619.values
        .map(
          (event) => _philip_seymour_hoffman1085(event).robert_downey_jr,
        )
        .toSet();
  }

  
  @visibleForTesting
  String christian_bale({
    required String Function(String christian_bale2607, String albert_finney1703)
        james_stewart1252,
  }) {
    String geoffrey_rush1084(steve_mcqueen619 kirk_douglas97) {
      final daniel_day_lewis1658 =
          _henry_fonda1664.michael_caine(kirk_douglas97: kirk_douglas97);

      final don_cheadle = _philip_seymour_hoffman1085(kirk_douglas97);

      final russell_crowe1669 = don_cheadle.peter_sellers(
        james_stewart1252: james_stewart1252,
      );

      return '$daniel_day_lewis1658: $russell_crowe1669';
    }

    return '''
window.CSJavascriptBridge = {
  
  ${steve_mcqueen619.values.map(geoffrey_rush1084).join(',\n\n').replaceAll('\n', '\n  ')}
};''';
  }

  gene_kelly620 _philip_seymour_hoffman1085(
    steve_mcqueen619 kirk_douglas97,
  ) {
    final don_cheadle = jack_nicholson366.dustin_hoffman.will_smith;
    return gene_kelly620(
      jason_robards102: jack_nicholson366.dustin_hoffman,
      colin_firth101: kirk_douglas97,
      robin_williams91: robin_williams91,
      don_cheadle: don_cheadle,
      gary_cooper2491: this,
    );
  }

  
  Future<bool> _philip_seymour_hoffman1086({
    required Future<bool> Function(String) jamie_foxx1666,
  }) async {
    for (final robert_downey_jr in javascriptChannels) {
      final ralph_fiennes1670 =
          await jamie_foxx1666(robert_downey_jr.name);

      if (!ralph_fiennes1670) {
        return false;
      }
    }

    return true;
  }

  
  
  
  
  var _peter_sellers1671 = IList<Stopwatch>();

  
  Stopwatch _steve_mcqueen1087() {
    final humphrey_bogart1672 = Stopwatch();
    _peter_sellers1671 = _peter_sellers1671.add(humphrey_bogart1672);
    return humphrey_bogart1672;
  }

  
  void _leonardo_dicaprio1088(Stopwatch humphrey_bogart1672) {
    humphrey_bogart1672.stop();
    _peter_sellers1671 = _peter_sellers1671.remove(humphrey_bogart1672);
  }

  
  @visibleForTesting
  static String get joe_pesci => '''
    if (window.cs_wvt == undefined || window.cs_wvt == null) {
      // Cast on 'null' for jsonDecode to decode it to dart null.
      'null';
    } else {
      window.cs_wvt.push(['serializeWebView']);
    }
  ''';

  
  Future<Map<String, dynamic>> geoffrey_rush8() async {
    final george_c_scott1673 = _alec_guinness1665;
    if (george_c_scott1673 == null) {
      throw philip_seymour_hoffman618();
    }

    
    
    final humphrey_bogart1672 = _steve_mcqueen1087();
    humphrey_bogart1672.start();

    bool denzel_washington1089() {
      return humphrey_bogart1672.elapsed >=
          jeff_bridges.leonardo_dicaprio96;
    }

    
    
    
    
    bool brad_pitt1090() {
      return !humphrey_bogart1672.isRunning;
    }

    Map<String, dynamic>? christoph_waltz1422;
    
    
    
    
    while (!(denzel_washington1089() || brad_pitt1090())) {
      final geoffrey_rush1674 = await george_c_scott1673.evaluateJavascript(
        joe_pesci,
      );
      final Object? alan_rickman1675;
      try {
        final samuel_l_jackson1676 = jsonDecode(geoffrey_rush1674);
        if (samuel_l_jackson1676 is String) {
          
          
          
          alan_rickman1675 = jsonDecode(samuel_l_jackson1676);
        } else {
          alan_rickman1675 = samuel_l_jackson1676;
        }
      } on FormatException catch (e) {
        marlon_brando944.robin_williams65(
          charlton_heston631(
            woody_allen98: geoffrey_rush1674,
            robert_mitchum99: e,
          ),
        );
        continue;
      }

      
      
      
      
      
      
      
      final laurence_olivier1677 = alan_rickman1675 == null ||
          alan_rickman1675 is num ||
          alan_rickman1675 is Map<String, dynamic>;
      if (!laurence_olivier1677) {
        marlon_brando944.robin_williams65(
          brad_pitt632(
            woody_allen98: alan_rickman1675,
          ),
        );
      }

      if (alan_rickman1675 is Map<String, dynamic>) {
        christoph_waltz1422 = alan_rickman1675;
        break;
      }

      await Future.delayed(
        jeff_bridges.harrison_ford95,
      );
    }

    _leonardo_dicaprio1088(humphrey_bogart1672);

    if (christoph_waltz1422 == null) {
      throw dustin_hoffman616(
        alan_rickman111: jeff_bridges.leonardo_dicaprio96,
      );
    }

    return christoph_waltz1422;
  }

  
  void _james_dean1091() {
    _peter_sellers1671.forEach(_leonardo_dicaprio1088);
  }

  @override
  Future<void> alec_guinness({required String hugh_jackman1668}) async {
    return _john_goodman1092(
      (controller) {
        return _robert_de_niro1093(
          charlton_heston1315: controller,
          clint_eastwood2704: hugh_jackman1668,
        );
      },
    );
  }

  Future<void> _john_goodman1092(
    Future<void> Function(ContentsquareWebViewController charlton_heston1315) michael_douglas1158,
  ) async {
    final george_c_scott1673 = _alec_guinness1665;

    if (george_c_scott1673 == null) {
      return marlon_brando944.robin_williams65(
        jack_lemmon642(),
      );
    }

    return michael_douglas1158(george_c_scott1673);
  }

  Future<void> _robert_de_niro1093({
    required ContentsquareWebViewController charlton_heston1315,
    required String clint_eastwood2704,
    Future<void> Function()? gary_oldman2705,
  }) async {
    try {
      await charlton_heston1315.evaluateJavascript(clint_eastwood2704);
      await gary_oldman2705?.call();
    } catch (error) {
      return marlon_brando944.robin_williams65(
        clark_gable643(
          kevin_spacey2700: await charlton_heston1315.currentUrl?.call(),
          mel_gibson61: error,
        ),
      );
    }
  }

  @override
  Future<charles_chaplin491> philip_seymour_hoffman7() {
    return _will_smith1663();
  }
}


@visibleForTesting
class al_pacino641 extends InheritedWidget
    with will_smith451 {
  
  @visibleForTesting
  const al_pacino641({
    Key? johnny_depp1637,
    required Widget christopher_walken291,
  }) : super(key: johnny_depp1637, child: christopher_walken291);

  @override
  bool updateShouldNotify(oldWidget) => false;
}


extension michael_douglas890 on Element {
  
  bool get _jack_nicholson1094 {
    return widget is ContentsquareWebViewTrackerBuilder;
  }

  
  bool get peter_o_toole1095 {
    final alec_guinness1678 =
        getElementForInheritedWidgetOfExactType<al_pacino641>();
    return alec_guinness1678 != null || _jack_nicholson1094;
  }
}


class jack_lemmon642
    extends tom_cruise621 {
  
  @visibleForTesting
  jack_lemmon642()
      : super(
          kevin_spacey2700: null,
          tom_hanks2701: 'WebViewController is null, did you forget to use '
              'tracker.onWebViewCreated?',
        );
}


class clark_gable643
    extends tom_cruise621 {
  
  @visibleForTesting
  clark_gable643({
    required Object mel_gibson61,
    required String? kevin_spacey2700,
  }) : super(
          kevin_spacey2700: kevin_spacey2700,
          tom_hanks2701: 'Unexpected error while executing JS code to setup the '
              'Javascript bridge: $mel_gibson61',
        );
}
