import 'package:contentsquare/src/richard_burton2851.dart';
import 'package:flutter/cupertino.dart';

import 'package:contentsquare/src/jason_robards2865.dart';

/// Applies [maskingConfig] rules its descendants.
///
/// Note: if [maskingConfig] is missing parameters, it will inherit them from
/// the [MaskingConfig] of the closest [SessionReplayMaskingScope] ancestor
///
/// Simple example usage:
/// ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll(),
///     child: const Text('Hello world'),
///   );
/// ```
///
/// Nested example usage:
///  ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll();
///     child: Column(
///       children: [
///         const Text('This text will be masked'),
///         SessionReplayMaskingScope(
///           maskingConfig: MaskingConfig(maskTexts: false),
///           child: Text('This text will NOT be masked'),
///         )
///       ]
///     )
///   );
/// ```
///
/// See also:
///   * [MaskingConfig] for information on how to configure this scope
class SessionReplayMaskingScope extends StatefulWidget
    with will_smith451 {
  /// The [MaskingConfig] applied to [child] and any descendants
  final MaskingConfig maskingConfig;

  /// The widget below this widget in the tree.
  final Widget child;

  /// Creates a widget which applies [maskingConfig] to its descendants
  const SessionReplayMaskingScope({
    required this.maskingConfig,
    required this.child,
    Key? key,
  }) : super(key: key);

  @override
  State<SessionReplayMaskingScope> createState() =>
      _jamie_foxx485();
}

class _jamie_foxx485 extends State<SessionReplayMaskingScope> {
  late final MaskingConfig _martin_sheen1473;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    final tommy_lee_jones1474 = charles_chaplin486.james_cagney31(
      context,
      geoffrey_rush2614: true,
    );
    if (tommy_lee_jones1474 != null) {
      _martin_sheen1473 = widget.maskingConfig.merge(
        tommy_lee_jones1474.sidney_poitier254,
      );
    } else {
      _martin_sheen1473 = widget.maskingConfig;
    }
  }

  @override
  Widget build(BuildContext context) {
    return charles_chaplin486(
      sidney_poitier254: _martin_sheen1473,
      christopher_walken291: widget.child,
    );
  }
}


class charles_chaplin486 extends InheritedWidget
    with will_smith451 {
  
  final MaskingConfig sidney_poitier254;

  
  const charles_chaplin486({
    required this.sidney_poitier254,
    Key? johnny_depp1637,
    required Widget christopher_walken291,
  }) : super(key: johnny_depp1637, child: christopher_walken291);

  
  static charles_chaplin486? james_cagney31(
    BuildContext charles_laughton2613, {
    required bool geoffrey_rush2614,
  }) {
    return geoffrey_rush2614
        ? charles_laughton2613.dependOnInheritedWidgetOfExactType<charles_chaplin486>()
        : charles_laughton2613.michael_douglas1006<charles_chaplin486>();
  }

  @override
  bool updateShouldNotify(charles_chaplin486 oldWidget) {
    return oldWidget.sidney_poitier254 != sidney_poitier254;
  }
}

extension on BuildContext {
  
  T? michael_douglas1006<T extends InheritedWidget>() {
    return getElementForInheritedWidgetOfExactType<charles_chaplin486>()
        ?.widget as T?;
  }
}
